module ABNF
  module Parser
    class Compiler
      attr_accessor :output
      attr_reader :tokens

      def initialize tokens, rule_list=nil
        @tokens = tokens
        @rule_list = rule_list
      end

      def self.build abnf, include_common: nil
        include_common ||= false

        if include_common
          common_rule_list = Parser::RuleList.common
        end

        tokenizer = Tokenizer.build abnf
        instance = new tokenizer, common_rule_list
        instance.push RuleList
        instance
      end

      def self.call *arguments
        instance = build *arguments
        instance.()
        instance.rule_list
      end

      def call
        tokens.each do |token|
          handle token
        end

        output
      end

      def current_frame
        stack.last
      end

      def handle token
        return unless current_frame
        current_frame.(token)
      end

      def pop rule
        frame = stack.pop

        self.output = rule if stack.empty?

        frame.exited rule
      end

      def push rule_builder_cls, &block
        rule_builder = rule_builder_cls.new self
        frame = Frame.new rule_builder, block
        stack.push frame
      end

      def root
        stack.first
      end

      def rule_list
        @rule_list ||= Parser::RuleList.new
      end

      def stack
        @stack ||= []
      end

      class Frame
        attr_reader :block
        attr_reader :rule_builder

        def initialize rule_builder, block
          @block = block
          @rule_builder = rule_builder
        end

        def call token
          rule_builder.(token)
        end

        def exited rule
          block.(rule) if block
        end
      end
    end
  end
end
