module ABNF
  module Parser
    class Compiler
      class Element < RuleBuilder
        attr_accessor :rule

        def bin_val_range token
          num_val_range token, 2
        end

        def bin_val_sequence token
          num_val_sequence token, 2
        end

        def char_val token
          string = token.lexeme[1...-1]
          rule = Rules::Terminal.string string, token.abnf
          compiler.pop rule
        end

        def dec_val_range token
          num_val_range token, 10
        end

        def dec_val_sequence token
          num_val_sequence token, 10
        end

        def group_start _
          compiler.push Group do |rule|
            compiler.pop rule
          end
        end

        def hex_val_range token
          num_val_range token, 16
        end

        def hex_val_sequence token
          num_val_sequence token, 16
        end

        def num_val_range token, base
          first, last = token.lexeme[2..-1].split '-'

          first = first.to_i base
          last = last.to_i base

          range = (first.chr..last.chr)

          rule = Rules::Terminal.character_range range, token.abnf
          compiler.pop rule
        end

        def num_val_sequence token, base
          codes = token.lexeme[2..-1].split '.'

          codes.map! do |code|
            code.to_i base
          end

          string = codes.pack 'C*'
          rule = Rules::Terminal.string string, token.abnf
          compiler.pop rule
        end

        def option_start _
          compiler.push Option do |rule|
            abnf = "[#{rule.abnf}]"
            rule = Rules::Repetition.new rule, (0..1), abnf
            compiler.pop rule
          end
        end

        def prose_val token
          rule = Rules::ProseVal.new token.abnf
          compiler.pop rule
        end

        def rulename token
          rule = Rules::Reference.new token.lexeme
          compiler.pop rule
        end
      end
    end
  end
end
