module ABNF
  module Parser
    class Compiler
      class RuleList < RuleBuilder
        attr_accessor :rulename

        def c_nl token
        end

        def coerce_alternation rule
          if rule.is_a? Rules::Alternation
            rule
          else
            Rules::Alternation.new [rule], rule.abnf
          end
        end

        def handle_rule name, rule
          actual_rule = rule.inner_rule

          if rule.operator == '=/'
            original_rule = coerce_alternation rule_list[name]
            new_rule = coerce_alternation actual_rule

            alternatives = original_rule.alternatives + new_rule.alternatives
            abnf = alternatives.map &:abnf
            abnf *= ' / '
            actual_rule = Rules::Alternation.new alternatives, abnf
          end

          rule_list[name] = actual_rule
        end

        def rule_list
          compiler.rule_list
        end

        def rulename token
          rulename = token.lexeme

          compiler.push Rule do |rule|
            handle_rule rulename, rule
          end
        end

        class Rule < RuleBuilder
          attr_accessor :operator
          attr_accessor :inner_rule

          def c_wsp _
          end

          def defined_as token
            self.operator = token.lexeme
          end

          def start_rule token
            compiler.push Alternation do |rule|
              self.inner_rule = rule
              compiler.pop self
            end
          end
        end
      end
    end
  end
end
