module ABNF
  module Parser
    module Controls
      module Compiler
        module Tokens
          extend self

          def alternation_slash
            Parser::Compiler::Token.new 'alternation-SLASH', ' / '
          end

          def bin_val_range
            Parser::Compiler::Token.new 'bin-val-RANGE', '%b100000-100001'
          end

          def bin_val_sequence
            Parser::Compiler::Token.new 'bin-val-SEQUENCE', '%b1100110.1101111.1101111'
          end

          def c_nl comment=nil
            abnf = "#{comment}\r\n"
            Parser::Compiler::Token.new 'c-nl', abnf
          end

          def c_wsp abnf=nil
            abnf ||= ' '
            Parser::Compiler::Token.new 'c-wsp', abnf
          end

          def char_val string=nil
            string ||= 'foo'
            Parser::Compiler::Token.new 'char-val', string.inspect
          end

          def dec_val_range
            Parser::Compiler::Token.new 'dec-val-RANGE', '%d32-33'
          end

          def dec_val_sequence
            Parser::Compiler::Token.new 'dec-val-SEQUENCE', '%d102.111.111'
          end

          def defined_as operator=nil
            operator ||= '='
            Parser::Compiler::Token.new 'defined-as', operator
          end

          def group_start
            Parser::Compiler::Token.new 'group-START', '( '
          end

          def group_stop
            Parser::Compiler::Token.new 'group-STOP', ' )'
          end

          def hex_val_range
            Parser::Compiler::Token.new 'hex-val-RANGE', '%x20-21'
          end

          def hex_val_sequence sequence=nil
            sequence ||= ABNF.foo
            Parser::Compiler::Token.new 'hex-val-SEQUENCE', sequence
          end
          alias_method :example, :hex_val_sequence
          alias_method :foo, :hex_val_sequence
          alias_method :num_val, :hex_val_sequence
          alias_method :terminal, :hex_val_sequence

          def option_start
            Parser::Compiler::Token.new 'option-START', '[ '
          end

          def option_stop
            Parser::Compiler::Token.new 'option-STOP', ' ]'
          end

          def prose_val
            Parser::Compiler::Token.new 'prose-val', '<Some Prose>'
          end

          def repeat_exact
            Parser::Compiler::Token.new 'repeat-EXACT', '1'
          end

          def repeat_range abnf=nil
            abnf ||= '*'
            Parser::Compiler::Token.new 'repeat-RANGE', abnf
          end

          def rulename
            Parser::Compiler::Token.new 'rulename', 'some-rule'
          end
        end
      end
    end
  end
end
