module ABNF
  module Parser
    module Controls
      module Rules
        extend self

        def alternation
          Parser::Rules::Alternation.new(
            [
              Parser::Rules::Terminal.string('foo', ABNF.foo),
              Parser::Rules::Terminal.string('bar', ABNF.bar),
              Parser::Rules::Terminal.string('baz', ABNF.baz),
            ],
            ABNF.alternation,
          )
        end

        def concatenation
          Parser::Rules::Concatenation.new(
            [
              Parser::Rules::Terminal.string('[', '"["'),
              Parser::Rules::Terminal.string('foo', ABNF.foo),
              Parser::Rules::Terminal.string(']', '"]"'),
            ],
            ABNF.concatenation,
          )
        end

        def option rule=nil
          rule ||= example
          abnf = "[#{rule.abnf}]"
          Parser::Rules::Repetition.new rule, (0..1), abnf
        end

        def prose_val
          Parser::Rules::ProseVal.new '<Some Prose>'
        end

        def reference
          Parser::Rules::Reference.new 'some-rule'
        end

        def regexp_pattern
          regexp = %r{(?<word1>[[:alpha:]]+)-(?<word2>[[:alpha:]]+)|~}
          Parser::Rules::RegexpPattern.new regexp, ABNF.regexp_pattern, 9
        end

        def repetition abnf: nil, range: nil
          range ||= (0..Float::INFINITY)
          abnf ||= '*"foo"'

          Parser::Rules::Repetition.new terminal, range, abnf
        end

        def terminal
          Parser::Rules::Terminal.string 'foo', ABNF.foo
        end
        alias_method :example, :terminal
        alias_method :foo, :terminal

        def terminal_range
          Parser::Rules::Terminal.character_range (' '..'!'), ABNF.foo
        end
      end
    end
  end
end
