module ABNF
  module Parser
    module Node
      def self.alternation node, abnf
        Alternation.new node, abnf
      end

      def self.concatenation children=nil, abnf
        children ||= Array(children)
        Sequence.new children, abnf
      end

      def self.repetition children=nil, abnf
        children ||= Array(children)
        Sequence.new children, abnf
      end

      def self.pattern_match match_data, abnf
        PatternMatch.new match_data, abnf
      end

      def self.terminal text, abnf
        Single.new text, abnf
      end

      def == other_node
        return false unless other_node.is_a? Node
        self.text == other_node.text and self.abnf == other_node.abnf
      end

      def [] index
        children[index]
      end

      def children
        []
      end

      def child_count
        children.size
      end

      def each &block
        children.each &block
      end

      def octets
        text.bytesize
      end

      class Alternation
        include Node

        attr_reader :abnf
        attr_reader :match

        def initialize match, abnf
          @abnf = abnf
          @match = match
        end

        def text
          match.text
        end
      end

      class Sequence
        include Enumerable
        include Node

        attr_reader :abnf
        attr_reader :children

        def initialize children, abnf
          @abnf = abnf
          @children = children
        end

        def add child
          children << child
        end
        alias_method :<<, :add

        def text
          @text ||= to_a.join
        end

        def to_a
          map do |child|
            if child.respond_to? :to_a
              child.to_a
            else
              child.text
            end
          end
        end
      end

      class Single
        include Node

        attr_reader :abnf
        attr_reader :text

        def initialize text, abnf
          @text = text
          @abnf = abnf
        end
      end

      class PatternMatch
        include Node

        attr_reader :abnf
        attr_reader :match_data

        def initialize match_data, abnf
          @match_data = match_data
          @abnf = abnf
        end

        def pattern
          match_data.regexp
        end

        def text
          match_data.to_s
        end

        def to_h
          hash = {}

          match_data.names.each do |name|
            hash[name] = match_data[name]
          end

          hash
        end
      end
    end
  end
end
