module ABNF
  module Parser
    class RuleList
      def == other_rule_list
        return false unless other_rule_list.is_a? self.class

        rules == other_rule_list.rules
      end

      def [] rule_name
        rules[rule_name]
      end

      def []= rule_name, rule
        rules[rule_name] = rule
      end

      def apply rule_name, io
        rule = fetch rule_name

        call_method = rule.method :call

        io = StringIO.new io if io.is_a? String

        rules[rule_name].(io, self)
      end

      def fetch rule_name
        rule = rules[rule_name]

        unless rule
          raise Error.new "Rule not found: #{rule_name.inspect}"
        end

        rule
      end

      def names
        rules.keys
      end

      def replace rule_name, regexp, max_octets
        existing_rule = fetch rule_name
        abnf = existing_rule

        rule = Rules::RegexpPattern.new regexp, abnf, max_octets
        rules[rule_name] = rule
      end

      def rules
        @rules ||= {}
      end

      def update additional_rules
        rules.update additional_rules
      end

      def self.common
        abnf = Common.to_s
        Compiler.(abnf)
      end
    end
  end
end
