module ABNF
  module Parser
    module Rules
      class Alternation
        attr_reader :abnf
        attr_reader :alternatives

        def initialize alternatives, abnf
          @abnf = abnf
          @alternatives = alternatives
        end

        def == other_rule
          return unless other_rule.is_a? self.class
          alternatives == other_rule.alternatives
        end

        def call io, rule_list=nil
          best_match = nil

          alternatives.each do |alternative|
            node = alternative.(io, rule_list) or next

            io.seek -node.octets, IO::SEEK_CUR

            best_match ||= node
            best_match = node if node.octets > best_match.octets
          end

          return unless best_match

          io.seek best_match.octets, IO::SEEK_CUR
          best_match
        end
      end
    end
  end
end
