module ABNF
  module Parser
    module Rules
      class Terminal
        attr_reader :abnf
        attr_reader :pattern

        def initialize pattern, abnf
          @abnf = abnf
          @pattern = pattern
          fail if self.class == Terminal
        end

        def == other_rule
          return false unless other_rule.is_a? self.class
          pattern == other_rule.pattern
        end

        def call io, _=nil
          potential_match = io.read octets
          return unless potential_match

          matches = match? potential_match

          if matches
            Node.terminal potential_match, abnf
          else
            io.seek -potential_match.bytesize, IO::SEEK_CUR
            nil
          end
        end

        class String < Terminal
          def match? potential_match
            pattern == potential_match
          end

          def octets
            pattern.bytesize
          end
        end

        def self.string *arguments
          String.new *arguments
        end

        class CharacterRange < Terminal
          def match? potential_match
            pattern.include? potential_match
          end

          def octets
            1
          end
        end

        def self.character_range *arguments
          CharacterRange.new *arguments
        end
      end
    end
  end
end
